@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009-2014 kaliko <kaliko@azylum.org>
+# Copyright (c) 2009-2014, 2021 kaliko <kaliko@azylum.org>
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@ __version__ = '0.5.1'
 __author__ = 'Jack Kaliko'
 
 
-
 from sima import LFM
 from sima.lib.meta import Artist
 from sima.lib.track import Track
@@ -47,9 +46,9 @@ class SimaFM:
 
     :param BaseCache cache: Set a cache, defaults to `False`.
     """
-    stats = {'etag':0,
-             'ccontrol':0,
-             'total':0}
+    stats = {'etag': 0,
+             'ccontrol': 0,
+             'total': 0}
 
     def __init__(self):
         self.http = HttpClient(cache=self.cache, stats=self.stats)
@@ -62,25 +61,25 @@ class SimaFM:
             code = ans.get('error')
             mess = ans.get('message')
             if code == 6:
-                raise WSNotFound('{0}: "{1}"'.format(mess, self.artist))
+                raise WSNotFound(f'{mess}: "{self.artist}"')
             raise WSError(mess)
         return True
 
     def _forge_payload(self, artist, method='similar', track=None):
         """Build payload
         """
-        payloads = dict({'similar': {'method':'artist.getsimilar',},
-                         'top': {'method':'artist.gettoptracks',},
-                         'track': {'method':'track.getsimilar',},
-                         'info': {'method':'artist.getinfo',},
-                        })
+        payloads = dict({'similar': {'method': 'artist.getsimilar',},
+                         'top': {'method': 'artist.gettoptracks',},
+                         'track': {'method': 'track.getsimilar',},
+                         'info': {'method': 'artist.getinfo',},
+                         })
         payload = payloads.get(method)
         payload.update(api_key=LFM.get('apikey'), format='json')
         if not isinstance(artist, Artist):
-            raise TypeError('"{0!r}" not an Artist object'.format(artist))
+            raise TypeError(f'"{artist!r}" not an Artist object')
         self.artist = artist
         if artist.mbid:
-            payload.update(mbid='{0}'.format(artist.mbid))
+            payload.update(mbid=f'{artist.mbid}')
         else:
             payload.update(artist=artist.name,
                            autocorrect=1)
@@ -104,18 +103,18 @@ class SimaFM:
             ans.json()
         except ValueError as err:
             # Corrupted/malformed cache? cf. gitlab issue #35
-            raise WSError('Malformed json, try purging the cache: %s')
-        self._controls_answer(ans.json()) # pylint: disable=no-member
+            raise WSError('Malformed json, try purging the cache: %s') from err
+        self._controls_answer(ans.json())  # pylint: disable=no-member
         # Artist might be found but return no 'artist' list…
         # cf. "Mulatu Astatqe" vs. "Mulatu Astatqé" with autocorrect=0
         # json format is broken IMHO, xml is more consistent IIRC
         # Here what we got:
         # >>> {"similarartists":{"#text":"\n","artist":"Mulatu Astatqe"}}
         # autocorrect=1 should fix it, checking anyway.
-        simarts = ans.json().get('similarartists').get('artist') # pylint: disable=no-member
+        simarts = ans.json().get('similarartists').get('artist')  # pylint: disable=no-member
         if not isinstance(simarts, list):
             raise WSError('Artist found but no similarities returned')
-        for art in ans.json().get('similarartists').get('artist'): # pylint: disable=no-member
+        for art in ans.json().get('similarartists').get('artist'):  # pylint: disable=no-member
             yield Artist(name=art.get('name'), mbid=art.get('mbid', None))
 
     def get_toptrack(self, artist):
@@ -126,8 +125,8 @@ class SimaFM:
         """
         payload = self._forge_payload(artist, method='top')
         ans = self.http(self.root_url, payload)
-        self._controls_answer(ans.json()) # pylint: disable=no-member
-        tops = ans.json().get('toptracks').get('track') # pylint: disable=no-member
+        self._controls_answer(ans.json())  # pylint: disable=no-member
+        tops = ans.json().get('toptracks').get('track')  # pylint: disable=no-member
         art = {'artist': artist.name,
                'musicbrainz_artistid': artist.mbid,}
         for song in tops:
