@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009-2020 kaliko <kaliko@azylum.org>
+# Copyright (c) 2009-2021 kaliko <kaliko@azylum.org>
 #
 #  This file is part of sima
 #
@@ -19,10 +19,11 @@
 # standard library import
 from difflib import get_close_matches
 from functools import wraps
-from itertools import dropwhile
+from logging import getLogger
+from select import select
 
 # external module
-from musicpd import MPDClient, MPDError
+from musicpd import MPDClient, MPDError as PlayerError
 
 
 # local import
@@ -32,10 +33,6 @@ from .lib.simastr import SimaStr
 from .utils.leven import levenshtein_ratio
 
 
-class PlayerError(Exception):
-    """Fatal error in the player."""
-
-
 # Some decorators
 def bl_artist(func):
     def wrapper(*args, **kwargs):
@@ -45,23 +42,31 @@ def bl_artist(func):
         result = func(*args, **kwargs)
         if not result:
             return None
-        names = list()
         for art in result.names:
-            if cls.database.get_bl_artist(art, add_not=True):
-                cls.log.debug('Blacklisted "%s"', art)
-                continue
-            names.append(art)
-        if not names:
-            return None
-        resp = Artist(name=names.pop(), mbid=result.mbid)
-        for name in names:
-            resp.add_alias(name)
-        return resp
+            artist = Artist(name=art, mbid=result.mbid)
+            if cls.database.get_bl_artist(artist, add=False):
+                cls.log.debug('Artist in blocklist: %s', artist)
+                return None
+        return result
+    return wrapper
+
+
+def set_artist_mbid(func):
+    def wrapper(*args, **kwargs):
+        cls = args[0]
+        result = func(*args, **kwargs)
+        if Meta.use_mbid:
+            if result and not result.mbid:
+                mbid = cls._find_musicbrainz_artistid(result)
+                artist = Artist(name=result.name, mbid=mbid)
+                artist.add_alias(result)
+                return artist
+        return result
     return wrapper
 
 
 def tracks_wrapper(func):
-    """Convert plain track mapping as returned by MPDClient into :py:obj:Track
+    """Convert plain track mapping as returned by MPDClient into :py:obj:`sima.lib.track.Track`
     objects. This decorator accepts single track or list of tracks as input.
     """
     @wraps(func)
@@ -74,42 +79,11 @@ def tracks_wrapper(func):
 # / decorators
 
 
-def blacklist(artist=False, album=False, track=False):
-    # pylint: disable=C0111,W0212
-    field = (album, track)
-
-    def decorated(func):
-        def wrapper(*args, **kwargs):
-            if not args[0].database:
-                return func(*args, **kwargs)
-            cls = args[0]
-            boolgen = (bl for bl in field)
-            bl_fun = (cls.database.get_bl_album,
-                      cls.database.get_bl_track,)
-            #bl_getter = next(fn for fn, bl in zip(bl_fun, boolgen) if bl is True)
-            bl_getter = next(dropwhile(lambda _: not next(boolgen), bl_fun))
-            #cls.log.debug('using {0} as bl filter'.format(bl_getter.__name__))
-            results = list()
-            for elem in func(*args, **kwargs):
-                if bl_getter(elem, add_not=True):
-                    #cls.log.debug('Blacklisted "{0}"'.format(elem))
-                    continue
-                if track and cls.database.get_bl_album(elem, add_not=True):
-                    # filter album as well in track mode
-                    # (artist have already been)
-                    cls.log.debug('Blacklisted alb. "%s"', elem)
-                    continue
-                results.append(elem)
-            return results
-        return wrapper
-    return decorated
-
-
 class MPD(MPDClient):
     """
     Player instance inheriting from MPDClient (python-musicpd).
 
-    Some methods are overridden to format objects as sima.lib.Track for
+    Some methods are overridden to format objects as :py:obj:`sima.lib.track.Track` for
     instance, other are calling parent class directly through super().
     cf. MPD.__getattr__
 
@@ -133,21 +107,24 @@ class MPD(MPDClient):
                           'MUSICBRAINZ_RELEASETRACKID', 'MUSICBRAINZ_WORKID'}
     database = None
 
-    def __init__(self, daemon):
+    def __init__(self, config):
         super().__init__()
+        self.socket_timeout = 10
         self.use_mbid = True
-        self.daemon = daemon
-        self.log = daemon.log
-        self.config = self.daemon.config['MPD']
+        self.log = getLogger('sima')
+        self.config = config
         self._cache = None
 
     # ######### Overriding MPDClient ###########
     def __getattr__(self, cmd):
         """Wrapper around MPDClient calls for abstract overriding"""
         track_wrapped = {'currentsong', 'find', 'playlistinfo', }
-        if cmd in track_wrapped:
-            return tracks_wrapper(super().__getattr__(cmd))
-        return super().__getattr__(cmd)
+        try:
+            if cmd in track_wrapped:
+                return tracks_wrapper(super().__getattr__(cmd))
+            return super().__getattr__(cmd)
+        except OSError as err:  # socket errors
+            raise PlayerError(err) from err
 
     def disconnect(self):
         """Overriding explicitly MPDClient.disconnect()"""
@@ -156,59 +133,57 @@ class MPD(MPDClient):
 
     def connect(self):
         """Overriding explicitly MPDClient.connect()"""
+        mpd_config = self.config['MPD']
         # host, port, password
-        host = self.config.get('host')
-        port = self.config.get('port')
-        password = self.config.get('password', fallback=None)
+        host = mpd_config.get('host')
+        port = mpd_config.get('port')
+        password = mpd_config.get('password', fallback=None)
         self.disconnect()
         try:
             super().connect(host, port)
         # Catch socket errors
-        except IOError as err:
-            raise PlayerError('Could not connect to "%s:%s": %s' %
-                              (host, port, err.strerror))
+        except OSError as err:
+            raise PlayerError(f'Could not connect to "{host}:{port}": {err.strerror}'
+                             ) from err
         # Catch all other possible errors
         # ConnectionError and ProtocolError are always fatal.  Others may not
         # be, but we don't know how to handle them here, so treat them as if
         # they are instead of ignoring them.
         except MPDError as err:
-            raise PlayerError('Could not connect to "%s:%s": %s' %
-                              (host, port, err))
+            raise PlayerError(f'Could not connect to "{host}:{port}": {err}') from err
         if password:
             try:
                 self.password(password)
-            except (MPDError, IOError) as err:
-                raise PlayerError("Could not connect to '%s': %s" % (host, err))
+            except OSError as err:
+                raise PlayerError(f"Could not connect to '{host}': {err}") from err
         # Controls we have sufficient rights
         available_cmd = self.commands()
         for cmd in MPD.needed_cmds:
             if cmd not in available_cmd:
                 self.disconnect()
-                raise PlayerError('Could connect to "%s", '
-                                  'but command "%s" not available' %
-                                  (host, cmd))
-        self.tagtypes('clear')
+                raise PlayerError(f'Could connect to "{host}", but command "{cmd}" not available')
+        self.tagtypes_clear()
         for tag in MPD.needed_tags:
-            self.tagtypes('enable', tag)
-        tt = set(map(str.lower, self.tagtypes()))
+            self.tagtypes_enable(tag)
+        ltt = set(map(str.lower, self.tagtypes()))
         needed_tags = set(map(str.lower, MPD.needed_tags))
-        if len(needed_tags & tt) != len(MPD.needed_tags):
-            self.log.warning('MPD exposes: %s', tt)
+        if len(needed_tags & ltt) != len(MPD.needed_tags):
+            self.log.warning('MPD exposes: %s', ltt)
             self.log.warning('Tags needed: %s', needed_tags)
             raise PlayerError('Missing mandatory metadata!')
         for tag in MPD.needed_mbid_tags:
-            self.tagtypes('enable', tag)
+            self.tagtypes_enable(tag)
         # Controls use of MusicBrainzIdentifier
-        if self.daemon.config.get('sima', 'musicbrainzid'):
-            tt = set(self.tagtypes())
-            if len(MPD.needed_mbid_tags & tt) != len(MPD.needed_mbid_tags):
+        if self.config.getboolean('sima', 'musicbrainzid'):
+            ltt = set(self.tagtypes())
+            if len(MPD.needed_mbid_tags & ltt) != len(MPD.needed_mbid_tags):
                 self.log.warning('Use of MusicBrainzIdentifier is set but MPD '
                                  'is not providing related metadata')
-                self.log.info(tt)
+                self.log.info(ltt)
                 self.log.warning('Disabling MusicBrainzIdentifier')
                 self.use_mbid = Meta.use_mbid = False
             else:
-                self.log.debug('Available metadata: %s', tt)
+                self.log.debug('Available metadata: %s', ltt)
                 self.use_mbid = Meta.use_mbid = True
         else:
             self.log.warning('Use of MusicBrainzIdentifier disabled!')
@@ -255,15 +230,22 @@ class MPD(MPDClient):
             * skipped   current track skipped
         """
         curr = self.current
-        try:
-            ret = self.idle('database', 'playlist', 'player', 'options')
-        except (MPDError, IOError) as err:
-            raise PlayerError("Couldn't init idle: %s" % err)
-        if self._skipped_track(curr):
-            ret.append('skipped')
-        if 'database' in ret:
-            self._reset_cache()
-        return ret
+        select_timeout = 5
+        try:  # noidle cmd does not go through __getattr__, need to catch OSError then
+            while True:
+                self.send_idle('database', 'playlist', 'player', 'options')
+                _read, _, _ = select([self], [], [], select_timeout)
+                if _read:  # tries to read response
+                    ret = self.fetch_idle()
+                    if self._skipped_track(curr):
+                        ret.append('skipped')
+                    if 'database' in ret:
+                        self._reset_cache()
+                    return ret
+                #  Nothing to read, canceling idle
+                self.noidle()
+        except OSError as err:
+            raise PlayerError(err) from err
 
     def clean(self):
         """Clean blocking event (idle) and pending commands
@@ -276,7 +258,7 @@ class MPD(MPDClient):
     def add(self, payload):
         """Overriding MPD's add method to accept Track objects
 
-        :param Track,list payload: Either a single :py:obj:`Track` or a list of it
+        :param Track,list payload: Either a single track or a list of it
         """
         if isinstance(payload, Track):
             super().__getattr__('add')(payload.file)
@@ -304,7 +286,7 @@ class MPD(MPDClient):
         plm = {'repeat': None, 'single': None,
                'random': None, 'consume': None, }
         for key, val in self.status().items():
-            if key in plm.keys():
+            if key in plm:
                 plm.update({key: bool(int(val))})
         return plm
 
@@ -325,11 +307,11 @@ class MPD(MPDClient):
     def find_tracks(self, what):
         """Find tracks for a specific artist or album
             >>> player.find_tracks(Artist('Nirvana'))
-            >>> player.find_tracks(Album('In Utero', artist=(Artist('Nirvana'))
+            >>> player.find_tracks(Album('In Utero', artist=Artist('Nirvana'))
 
         :param Artist,Album what: Artist or Album to fetch track from
-
-        Returns a list of :py:obj:Track objects
+        :return: A list of track objects
+        :rtype: list(Track)
         """
         if isinstance(what, Artist):
             return self._find_art(what)
@@ -341,32 +323,79 @@ class MPD(MPDClient):
 
     def _find_art(self, artist):
         tracks = set()
+        # artist blocklist
+        if self.database.get_bl_artist(artist, add=False):
+            self.log.info('Artist in blocklist: %s', artist)
+            return []
         if artist.mbid:
             tracks |= set(self.find('musicbrainz_artistid', artist.mbid))
-        for name in artist.names_sz:
+        for name in artist.names:
             tracks |= set(self.find('artist', name))
+        # album blocklist
+        albums = {Album(trk.Album.name, mbid=trk.musicbrainz_albumid)
+                  for trk in tracks}
+        bl_albums = {Album(a.get('album'), mbid=a.get('musicbrainz_album'))
+                     for a in self.database.view_bl() if a.get('album')}
+        if albums & bl_albums:
+            self.log.info('Albums in blocklist for %s: %s', artist, albums & bl_albums)
+            tracks = {trk for trk in tracks if trk.Album not in bl_albums}
+        # track blocklist
+        bl_tracks = {Track(title=t.get('title'), file=t.get('file'))
+                     for t in self.database.view_bl() if t.get('title')}
+        if tracks & bl_tracks:
+            self.log.info('Tracks in blocklist for %s: %s',
+                          artist, tracks & bl_tracks)
+            tracks = {trk for trk in tracks if trk not in bl_tracks}
         return list(tracks)
 
     def _find_alb(self, album):
         if not hasattr(album, 'artist'):
             raise PlayerError('Album object have no artist attribute')
+        if self.database.get_bl_album(album, add=False):
+            self.log.info('Album in blocklist: %s', album)
+            return []
         albums = []
-        if self.use_mbid and album.mbid:
+        if album.mbid:
             filt = f"(MUSICBRAINZ_ALBUMID == '{album.mbid}')"
             albums = self.find(filt)
         # Now look for album with no MusicBrainzIdentifier
-        if not albums and album.artist.mbid and self.use_mbid:  # Use album artist MBID if possible
-            filt = f"((MUSICBRAINZ_ALBUMARTISTID == '{album.artist.mbid}') AND (album == '{album.name_sz}'))"
+        if not albums and album.Artist.mbid:  # Use album artist MBID if possible
+            filt = f"((MUSICBRAINZ_ALBUMARTISTID == '{album.Artist.mbid}') AND (album == '{album.name_sz}'))"
             albums = self.find(filt)
         if not albums:  # Falls back to (album)?artist/album name
-            for artist in album.artist.names_sz:
+            for artist in album.Artist.names_sz:
                 filt = f"((albumartist == '{artist}') AND (album == '{album.name_sz}'))"
                 albums.extend(self.find(filt))
         return albums
 # #### / find_tracks ##
 
 # #### Search Methods #####
+    def _find_musicbrainz_artistid(self, artist):
+        """Find MusicBrainzArtistID when possible.
+        """
+        if not self.use_mbid:
+            return None
+        mbids = None
+        for name in artist.names_sz:
+            filt = f'((artist == "{name}") AND (MUSICBRAINZ_ARTISTID != ""))'
+            mbids = self.list('MUSICBRAINZ_ARTISTID', filt)
+            if mbids:
+                break
+        if not mbids:
+            return None
+        if len(mbids) > 1:
+            self.log.debug("Got multiple MBID for artist: %r", artist)
+            return None
+        if artist.mbid:
+            if artist.mbid != mbids[0]:
+                self.log('MBID discrepancy, %s found with %s (instead of %s)',
+                         artist.name, mbids[0], artist.mbid)
+        else:
+            return mbids[0]
+        return None
+
     @bl_artist
+    @set_artist_mbid
     def search_artist(self, artist):
         """
         Search artists based on a fuzzy search in the media library
@@ -376,11 +405,11 @@ class MPD(MPDClient):
             >>> ['The Beatles', 'Beatles', 'the beatles']
 
         :param Artist artist: Artist to look for in MPD music library
-
-        Returns an Artist object
+        :return: Artist object
+        :rtype: Artist
         """
         found = False
-        if self.use_mbid and artist.mbid:
+        if artist.mbid:
             # look for exact search w/ musicbrainz_artistid
             library = self.list('artist', f"(MUSICBRAINZ_ARTISTID == '{artist.mbid}')")
             if library:
@@ -389,6 +418,10 @@ class MPD(MPDClient):
                 # library could fetch several artist name for a single MUSICBRAINZ_ARTISTID
                 if len(library) > 1:
                     self.log.debug('I got "%s" searching for %r', library, artist)
+                    for name in library:
+                        if SimaStr(artist.name) == name and name != artist.name:
+                            self.log.debug('add alias for %s: %s', artist, name)
+                            artist.add_alias(name)
                 elif len(library) == 1 and library[0] != artist.name:
                     new_alias = artist.name
                     self.log.info('Update artist name %s->%s', artist, library[0])
@@ -436,7 +469,6 @@ class MPD(MPDClient):
             return artist
         return None
 
-    @blacklist(track=True)
     def search_track(self, artist, title):
         """Fuzzy search of title by an artist
         """
@@ -466,7 +498,6 @@ class MPD(MPDClient):
                                mtitle, title, leven)
         return tracks
 
-    @blacklist(album=True)
     def search_albums(self, artist):
         """Find potential albums for "artist"
 
@@ -474,37 +505,69 @@ class MPD(MPDClient):
           → falls back to "Artist" == artist when no "AlbumArtist" tag is set
         * Tries to filter some mutli-artists album
           For instance an album by Artist_A may have a track by Artist_B. Then
-          looking for albums for Artist_B returns wrongly this album.
+          looking for albums for Artist_B wrongly returns this album.
         """
         # First, look for all potential albums
         self.log.debug('Searching album for "%r"', artist)
         if artist.aliases:
             self.log.debug('Searching album for %s aliases: "%s"',
                            artist, artist.aliases)
+        albums = set()
+        if self.use_mbid and artist.mbid:
+            mpd_filter = f"((musicbrainz_albumartistid == '{artist.mbid}') AND ( album != ''))"
+            raw_album_id = self.list('musicbrainz_albumid', mpd_filter)
+            for albumid in raw_album_id:
+                mpd_filter = f"((musicbrainz_albumid == '{albumid}') AND ( album != ''))"
+                album_name = self.list('album', mpd_filter)
+                if not album_name:  # something odd here
+                    continue
+                albums.add(Album(album_name[0], artist=artist.name,
+                                 Artist=artist, mbid=albumid))
         for name_sz in artist.names_sz:
             mpd_filter = f"((albumartist == '{name_sz}') AND ( album != ''))"
             raw_albums = self.list('album', mpd_filter)
-            albums = [Album(a, albumartist=artist.name, artist=artist) for a in raw_albums]
+            for alb in raw_albums:
+                if alb in [a.name for a in albums]:
+                    continue
+                mbid = None
+                if self.use_mbid:
+                    _ = Album(alb)
+                    mpd_filter = f"((albumartist == '{artist.name_sz}') AND ( album == '{_.name_sz}'))"
+                    mbids = self.list('MUSICBRAINZ_ALBUMID', mpd_filter)
+                    if mbids:
+                        mbid = mbids[0]
+                albums.add(Album(alb, artist=artist.name,
+                                 Artist=artist, mbid=mbid))
         candidates = []
         for album in albums:
             album_trks = self.find_tracks(album)
+            if not album_trks:  # find_track result can be empty, blocklist applied
+                continue
             album_artists = {tr.albumartist for tr in album_trks if tr.albumartist}
-            if album.artist.names & album_artists:
+            if album.Artist.names & album_artists:
                 candidates.append(album)
                 continue
+            if self.use_mbid and artist.mbid:
+                if artist.mbid == album_trks[0].musicbrainz_albumartistid:
+                    candidates.append(album)
+                    continue
+                self.log.debug('Discarding "%s", "%r" not set as musicbrainz_albumartistid',
+                               album, album.Artist)
+                continue
             if 'Various Artists' in album_artists:
                 self.log.debug('Discarding %s ("Various Artists" set)', album)
                 continue
-            if album_artists and album.artist.name not in album_artists:
-                self.log.debug('Discarding "%s", "%s" not set as albumartist', album, album.artist)
+            if album_artists and album.Artist.name not in album_artists:
+                self.log.debug('Discarding "%s", "%s" not set as albumartist', album, album.Artist)
                 continue
-            # Attempt to detect false positive
+            # Attempt to detect false positive (especially when no
+            # AlbumArtist/MBIDs tag ar set)
             # Avoid selecting albums where artist is credited for a single
             # track of the album
             album_trks = self.find(f"(album == '{album.name_sz}')")
             arts = [trk.artist for trk in album_trks]  # Artists in the album
             # count artist occurences
-            ratio = arts.count(album.artist.name)/len(arts)
+            ratio = arts.count(album.Artist.name)/len(arts)
             if ratio >= 0.8:
                 candidates.append(album)
             else:
